@@ -1,8 +1,7 @@
-# This file is part of GNU Health.  The COPYRIGHT file at the top level of
+# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import gobject
 import gtk
-import pango
 import sys
 import json
 import locale
@@ -12,11 +11,11 @@ from collections import defaultdict
 
 from tryton.config import CONFIG
 from tryton.common.cellrendererbutton import CellRendererButton
-from tryton.common.cellrenderertoggle import CellRendererToggle
 from tryton.gui.window import Window
 from tryton.common.popup_menu import populate
 from tryton.common import RPCExecute, RPCException, node_attributes, Tooltips
 from tryton.common import domain_inversion, simplify, unique_value
+from tryton.common.widget_style import widget_class
 from tryton.pyson import PYSONDecoder
 import tryton.common as common
 from . import View
@@ -274,14 +273,17 @@ class ViewTree(View):
         self.sum_widgets = []
         self.sum_box = gtk.HBox()
         self.reload = False
-        if self.attributes.get('editable'):
+        if self.attributes.get('editable') and not screen.readonly:
             self.treeview = EditableTreeView(self.attributes['editable'], self)
+            grid_lines = gtk.TREE_VIEW_GRID_LINES_BOTH
         else:
             self.treeview = TreeView(self)
+            grid_lines = gtk.TREE_VIEW_GRID_LINES_VERTICAL
 
         self.parse(xml)
 
         self.treeview.set_property('rules-hint', True)
+        self.treeview.set_property('enable-grid-lines', grid_lines)
         self.treeview.set_fixed_height_mode(
             all(c.get_sizing() == gtk.TREE_VIEW_COLUMN_FIXED
                 for c in self.treeview.get_columns()))
@@ -386,7 +388,7 @@ class ViewTree(View):
             column.set_cell_data_func(suffix.renderer,
                 suffix.setter)
 
-        self.set_column_widget(column, field, node_attrs)
+        self.set_column_widget(column, field, node_attrs, align=widget.align)
         self.set_column_width(column, field, node_attrs)
 
         if (not self.attributes.get('sequence')
@@ -448,29 +450,27 @@ class ViewTree(View):
     def get_widget(cls, name):
         return cls.WIDGETS[name]
 
-    def set_column_widget(self, column, field, attributes, arrow=True):
-        tooltips = Tooltips()
+    def set_column_widget(self, column, field, attributes,
+            arrow=True, align=0.5):
         hbox = gtk.HBox(False, 2)
         label = gtk.Label(attributes['string'])
         if field and self.editable:
             required = field.attrs.get('required')
             readonly = field.attrs.get('readonly')
-            if (required or not readonly) and hasattr(pango, 'AttrWeight'):
-                # FIXME when Pango.attr_weight_new is introspectable
-                attrlist = pango.AttrList()
-                if required:
-                    attrlist.insert(pango.AttrWeight(pango.WEIGHT_BOLD, 0, -1))
-                if not readonly:
-                    attrlist.change(pango.AttrStyle(pango.STYLE_ITALIC, 0, -1))
-                label.set_attributes(attrlist)
+            attrlist = common.get_label_attributes(readonly, required)
+            label.set_attributes(attrlist)
+            widget_class(label, 'readonly', readonly)
+            widget_class(label, 'required', required)
         label.show()
-        help = attributes['string']
+        help = None
         if field and field.attrs.get('help'):
-            help += '\n' + field.attrs['help']
+            help = field.attrs['help']
         elif attributes.get('help'):
-            help += '\n' + attributes['help']
-        tooltips.set_tip(label, help)
-        tooltips.enable()
+            help = attributes['help']
+        if help:
+            tooltips = Tooltips()
+            tooltips.set_tip(label, help)
+            tooltips.enable()
         if arrow:
             arrow_widget = gtk.Arrow(gtk.ARROW_NONE, gtk.SHADOW_NONE)
             arrow_widget.show()
@@ -481,7 +481,7 @@ class ViewTree(View):
             column.set_clickable(True)
         hbox.show()
         column.set_widget(hbox)
-        column.set_alignment(0.5)
+        column.set_alignment(align)
 
     def set_column_width(self, column, field, attributes):
         default_width = {
@@ -518,7 +518,7 @@ class ViewTree(View):
         expand = attributes.get('expand', False)
         column.set_expand(expand)
         column.set_resizable(True)
-        if not field or field.attrs['type'] != 'text':
+        if attributes.get('widget') != 'text':
             column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
 
     def get_column_widget(self, column):
@@ -627,7 +627,8 @@ class ViewTree(View):
 
     @property
     def editable(self):
-        return bool(getattr(self.treeview, 'editable', False))
+        return (bool(getattr(self.treeview, 'editable', False))
+            and not self.screen.readonly)
 
     def get_fields(self):
         return [col.name for col in self.treeview.get_columns() if col.name]
@@ -774,7 +775,8 @@ class ViewTree(View):
 
         def _func_sel_get(model, path, iter_, data):
             value = model.get_value(iter_, 0)
-            data.append(json.dumps(value.get_path(model.group)))
+            data.append(json.dumps(
+                    value.get_path(model.group), separators=(',', ':')))
         data = []
         treeselection = treeview.get_selection()
         treeselection.selected_foreach(_func_sel_get, data)
@@ -828,7 +830,7 @@ class ViewTree(View):
             return True
         if drop_info:
             path, position = drop_info
-            check_path = tuple(path)
+            check_path = path
             if position in (gtk.TREE_VIEW_DROP_BEFORE,
                     gtk.TREE_VIEW_DROP_AFTER):
                 check_path = path[:-1]
@@ -906,7 +908,10 @@ class ViewTree(View):
                     if not model:
                         continue
                     label = field.attrs['string']
-                    populate(menu, model, record_id, title=label, field=field)
+                    context = field.get_context(record)
+                    populate(
+                        menu, model, record_id, title=label, field=field,
+                        context=context)
                 menu.show_all()
 
             selection = treeview.get_selection()
@@ -1183,19 +1188,6 @@ class ViewTree(View):
         sel.selected_foreach(_func_sel_get, records)
         return records
 
-    def unset_editable(self):
-        self.treeview.editable = False
-        for col in self.treeview.get_columns():
-            for renderer in col.get_cell_renderers():
-                if isinstance(renderer, CellRendererToggle):
-                    renderer.set_property('activatable', False)
-                elif isinstance(renderer,
-                        (gtk.CellRendererProgress, CellRendererButton,
-                            gtk.CellRendererPixbuf)):
-                    pass
-                else:
-                    renderer.set_property('editable', False)
-
     def get_selected_paths(self):
         selection = self.treeview.get_selection()
         model, rows = selection.get_selected_rows()
